"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const chalk_1 = require("chalk");
const ECR = require("aws-sdk/clients/ecr");
const S3 = require("aws-sdk/clients/s3");
const api_1 = require("../../lib/api");
const aws_1 = require("../../lib/aws");
const cordova_1 = require("../../lib/cordova");
const docker_1 = require("../../lib/docker");
const fs_1 = require("../../lib/fs");
const npm_1 = require("../../lib/npm");
const shell_1 = require("../../lib/shell");
async function run(ctx) {
    await prepareDocker(ctx);
}
exports.run = run;
async function prepareDocker(ctx) {
    const image = ctx.env.require('PACKAGE_ANDROID_IMAGE');
    const snapshotBucket = ctx.env.require('SNAPSHOT_S3_BUCKET');
    const packageBucket = ctx.env.require('PACKAGE_S3_BUCKET');
    const snapshotId = ctx.env.require('SNAPSHOT_ID');
    const s3 = new aws_1.S3Client(new S3({
        apiVersion: '2006-03-01',
        accessKeyId: ctx.env.require('AWS_ACCESS_KEY_ID'),
        secretAccessKey: ctx.env.require('AWS_SECRET_ACCESS_KEY'),
        sessionToken: ctx.env.require('AWS_SESSION_TOKEN'),
    }));
    const ecr = new aws_1.ECRClient(new ECR({
        apiVersion: '2015-09-21',
        accessKeyId: ctx.env.require('AWS_ACCESS_KEY_ID'),
        secretAccessKey: ctx.env.require('AWS_SECRET_ACCESS_KEY'),
        sessionToken: ctx.env.require('AWS_SESSION_TOKEN'),
        region: ctx.env.require('AWS_REGION'),
    }));
    const token = await ecr.getLoginCredentials();
    await shell_1.runcmd({ outlog: ctx.serverlog }, 'docker', ['login', '--username', token.username, '--password', token.password, token.endpoint], {});
    ctx.userlog.info('Fetching build dependencies...');
    const cachehit = await npm_1.attemptCacheRestore(ctx);
    if (!cachehit) {
        await docker_1.prettyUserCommand(ctx, image, ['npm', 'install', '--quiet', '--no-optional']);
        await npm_1.cacheNodeModules(ctx);
    }
    try {
        ctx.serverlog.info(`Pulling ${image}...`);
        await shell_1.runcmd({ outlog: ctx.serverlog }, 'docker', ['pull', image], {});
    }
    catch (e) {
        ctx.serverlog.error(e);
        ctx.userlog.error('Failed to use cached dependencies, moving on...');
    }
    const filename = `${snapshotId}.zip`;
    const wwwpath = path.resolve(ctx.codedir, 'www');
    const zippath = path.resolve(ctx.codedir, filename);
    try {
        await fs_1.fsMkdir(wwwpath);
    }
    catch (e) {
        if (e.code !== 'EEXIST') {
            ctx.serverlog.error(e);
        }
    }
    const ws = fs.createWriteStream(zippath);
    await s3.download(snapshotBucket, filename, ws);
    await shell_1.runcmd({ outlog: ctx.serverlog }, 'unzip', ['-qo', zippath], { cwd: wwwpath });
    await fs_1.fsUnlink(zippath);
    const appId = ctx.env.require('APP_ID');
    const profileTag = ctx.env.get('CREDENTIAL_PROFILE_TAG');
    if (profileTag) {
        const credentials = (await api_1.getAndroidCredentials(ctx, appId, profileTag)).data;
        await cordova_1.cordovaPlatformAdd(ctx, image, 'android');
        await cordova_1.cordovaBuildAndroid(ctx, image, { credentials });
    }
    else {
        // debug build
    }
    const buildType = ctx.env.get('BUILD_TYPE', 'debug');
    const appName = ctx.env.require('APP_NAME');
    const branch = ctx.env.require('BRANCH');
    const sha = ctx.env.require('COMMIT_SHA');
    const artifactPath = path.resolve(ctx.codedir, `platforms/android/build/outputs/apk/android-${buildType}.apk`);
    const artifact = fs.createReadStream(artifactPath);
    const key = `${snapshotId}-${buildType}.apk`;
    const contentDisposition = `attachment; filename=${appName}-${branch}-${sha}-${buildType}.apk`;
    ctx.serverlog.info({ bucket: packageBucket, key }, `Uploading ${artifactPath} to s3://${packageBucket}/${key}`);
    ctx.userlog.info('Uploading apk...');
    await s3.upload(packageBucket, key, artifact, { ContentDisposition: contentDisposition });
    ctx.userlog.info(chalk_1.default.green('Uploaded apk successfully.'));
}
exports.prepareDocker = prepareDocker;
