"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const chalk_1 = require("chalk");
const archiver = require("archiver");
const S3 = require("aws-sdk/clients/s3");
const aws_1 = require("../../lib/aws");
const docker_1 = require("../../lib/docker");
const project_1 = require("../../lib/project");
const shell_1 = require("../../lib/shell");
const npm_1 = require("../../lib/npm");
async function run(ctx) {
    await prepareDocker(ctx);
    await buildApp(ctx);
    await uploadApp(ctx);
}
exports.run = run;
async function prepareDocker(ctx) {
    const environment = ctx.env.require('ENVIRONMENT');
    const dockerImage = ctx.env.require('SNAPSHOT_IMAGE');
    const dir = path.dirname(path.dirname(ctx.bindir));
    ctx.userlog.info('Fetching build dependencies...');
    await shell_1.runcmd({ outlog: ctx.serverlog }, 'gitlab-login', [], {});
    if (environment === 'DEVELOPMENT') {
        const dockerDir = path.resolve(dir, 'docker', 'snapshot');
        await shell_1.runcmd({ outlog: ctx.serverlog }, 'docker', ['build', '-t', dockerImage, dockerDir], { cwd: ctx.codedir });
    }
    else {
        await shell_1.runcmd({ outlog: ctx.serverlog }, 'docker', ['pull', dockerImage], { cwd: ctx.codedir });
    }
}
exports.prepareDocker = prepareDocker;
async function buildApp(ctx) {
    const image = ctx.env.require('SNAPSHOT_IMAGE');
    const dockerEnv = { APP_ID: ctx.env.require('APP_ID'), SNAPSHOT_ID: ctx.env.require('SNAPSHOT_ID') };
    const project = await project_1.readProjectFile(ctx);
    const cachehit = await npm_1.attemptCacheRestore(ctx);
    if (!cachehit) {
        await docker_1.prettyUserCommand(ctx, image, ['npm', 'install', '--quiet', '--no-optional'], dockerEnv);
        await npm_1.cacheNodeModules(ctx);
    }
    try {
        await docker_1.prettyUserCommand(ctx, image, ['npm', 'run', 'build'], dockerEnv);
    }
    catch (e) {
        if (project.type !== 'ionic1') {
            throw e;
        }
    }
}
exports.buildApp = buildApp;
async function uploadApp(ctx) {
    const s3 = new aws_1.S3Client(new S3({
        apiVersion: '2006-03-01',
        accessKeyId: ctx.env.require('AWS_ACCESS_KEY_ID'),
        secretAccessKey: ctx.env.require('AWS_SECRET_ACCESS_KEY'),
        sessionToken: ctx.env.require('AWS_SESSION_TOKEN'),
    }));
    const bucket = ctx.env.require('SNAPSHOT_S3_BUCKET');
    const snapshotId = ctx.env.require('SNAPSHOT_ID');
    ctx.userlog.info('Uploading build...');
    try {
        const archive = archiver('zip');
        const wwwpath = path.resolve(ctx.codedir, 'www');
        archive.directory(wwwpath, '');
        archive.finalize();
        await s3.upload(bucket, `${snapshotId}.zip`, archive);
        ctx.userlog.info(chalk_1.default.green('Uploaded build successfully.'));
    }
    catch (e) {
        ctx.userlog.error(chalk_1.default.red('Failed to upload build to storage. Please retry your build.'));
        throw e;
    }
}
exports.uploadApp = uploadApp;
