"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const crypto = require("crypto");
const path = require("path");
const archiver = require("archiver");
const tar = require("tar");
const S3 = require("aws-sdk/clients/s3");
const guards_1 = require("../guards");
const aws_1 = require("./aws");
const fs_1 = require("./fs");
const string_1 = require("./string");
exports.PACKAGE_JSON_FILE = 'package.json';
/**
 * The idea here is to create a unique hash for a set of dependencies in a
 * project.
 */
async function hashPackageJson(pkg) {
    const isEmpty = await Promise.resolve().then(() => require('lodash/isEmpty'));
    const hash = crypto.createHash('sha256');
    const extractDeps = (obj) => {
        return Object.entries(obj).sort(([k1,], [k2,]) => string_1.strcmp(k1, k2)).map(([k, v]) => `${k}->${v}`);
    };
    hash.update('dan is awesome');
    if (isEmpty(pkg.dependencies) && isEmpty(pkg.devDependencies)) {
        // No dependencies and no devDependencies is scary: hash the entire file.
        hash.update(JSON.stringify(pkg));
    }
    if (pkg.scripts) {
        const deps = extractDeps(pkg.scripts);
        hash.update(`scripts:${deps.join(',')}`);
    }
    if (pkg.dependencies) {
        const deps = extractDeps(pkg.dependencies);
        hash.update(`dependencies:${deps.join(',')}`);
    }
    if (pkg.devDependencies) {
        const deps = extractDeps(pkg.devDependencies);
        hash.update(`devDependencies:${deps.join(',')}`);
    }
    return hash.digest('hex');
}
exports.hashPackageJson = hashPackageJson;
async function attemptCacheRestore(ctx) {
    const s3 = new aws_1.S3Client(new S3({
        apiVersion: '2006-03-01',
        accessKeyId: ctx.env.require('AWS_ACCESS_KEY_ID'),
        secretAccessKey: ctx.env.require('AWS_SECRET_ACCESS_KEY'),
        sessionToken: ctx.env.require('AWS_SESSION_TOKEN'),
    }));
    const bucket = ctx.env.require('NM_CACHE_S3_BUCKET');
    const packageJson = await readPackageJson(ctx.codedir);
    const projectHash = await hashPackageJson(packageJson);
    ctx.userlog.info('Attempting to restore cached packages...');
    try {
        const extract = tar.x({ cwd: ctx.codedir });
        await s3.download(bucket, `${projectHash}.tar.gz`, extract);
    }
    catch (e) {
        if (e === aws_1.S3_NO_SUCH_KEY) {
            ctx.userlog.info('No cache found for packages, installing...');
        }
        else {
            ctx.serverlog.error(e);
            return false;
        }
    }
    return true;
}
exports.attemptCacheRestore = attemptCacheRestore;
async function cacheNodeModules(ctx) {
    const s3 = new aws_1.S3Client(new S3({
        apiVersion: '2006-03-01',
        accessKeyId: ctx.env.require('AWS_ACCESS_KEY_ID'),
        secretAccessKey: ctx.env.require('AWS_SECRET_ACCESS_KEY'),
        sessionToken: ctx.env.require('AWS_SESSION_TOKEN'),
    }));
    const bucket = ctx.env.require('NM_CACHE_S3_BUCKET');
    const packageJson = await readPackageJson(ctx.codedir);
    const projectHash = await hashPackageJson(packageJson);
    const archive = archiver('tar', { gzip: true });
    archive.directory(path.resolve(ctx.codedir, 'node_modules'), 'node_modules');
    archive.finalize();
    ctx.userlog.info('Saving package cache...');
    try {
        await s3.upload(bucket, `${projectHash}.tar.gz`, archive);
    }
    catch (e) {
        ctx.serverlog.error(e);
        ctx.userlog.error('Failed to upload package cache, moving on...');
    }
}
exports.cacheNodeModules = cacheNodeModules;
async function readPackageJson(dir) {
    try {
        const f = await fs_1.fsReadFile(path.resolve(dir, exports.PACKAGE_JSON_FILE), { encoding: 'utf8' });
        const o = JSON.parse(f);
        if (!guards_1.isPackageJson(o)) {
            throw new Error(`Invalid format in ${exports.PACKAGE_JSON_FILE}.`);
        }
        return o;
    }
    catch (e) {
        if (e.code === 'ENOENT') {
            throw new Error(`Missing ${exports.PACKAGE_JSON_FILE} in app.`);
        }
        else if (e instanceof SyntaxError) {
            throw new Error(`Invalid JSON in ${exports.PACKAGE_JSON_FILE}.`);
        }
        throw e;
    }
}
exports.readPackageJson = readPackageJson;
